  s" sendmailpreamble.frt" included
  s" unix/socket.fs"       included

 \  Demonstrates using socket API functions to perform email
 \  transmission using the SMTP protocol. Takes four parameters, as
 \  follows:

 \     sendmail "mailserver" "to_addr" "subject" "messagefile.txt"

 \  Connects to the specified mail server, uses to_addr (and an
 \  internally preset from_addr) in the transmitted message header,
 \  then sends real name (internal), subject, repeated To-field and
 \  each line of the messagefile (text file) as the email message
 \  body. Transfer is implemented using the SMTP Internet protocol.

 \  The message data is prepended with some extra info to prevent
 \  anti-spam software from dumping the mail immediately.
 

DECIMAL
25 VALUE IPPORT_SMTP                                 \ SMTP standard port address

2VARIABLE realname S" Marcel Hendrix" realname 2!
2VARIABLE msubject                                   \ msg subject
2VARIABLE mto                                        \ to address
2VARIABLE mfrom    S" mhx@iae.nl"     mfrom    2!    \ from address
2VARIABLE server                                     \ {name | address in dotted notation} of the
                                                     \ server or host system we need to connect to
0  VALUE msocket    
0  VALUE mfile      

: .data-header ( -- c-addr u ) 
	S" From: " >STR realname 2@ STR+
	S"  <"        STR+    mfrom    2@ STR+ S" >" STR+ +CR
	S" Subject: " STR+    msubject 2@ STR+ +CR
	S" To: "      STR+    mto      2@ STR+ +CR ;

: mail-init ( mailserver-addr&len to-addr&len subject-addr&len filename-addr&len -- )
	R/O BIN OPEN-FILE ?FILE TO mfile
	msubject 2!
	mto      2!
	server   2!
	server 2@ IPPORT_SMTP OPEN-SERVICE TO msocket
	msocket TRUE BLOCKING-MODE   ( blocking for mail)
	debug? IF  1000  ELSE 0 THEN SET-SOCKET-TIMEOUT
	msocket PAD 4096 READ-SOCKET .debug 2DROP ;

: sline ( c-addr u -- )
        +CR .debug
        msocket WRITE-SOCKET 
        msocket PAD 4096 READ-SOCKET .debug 2DROP ;

: helo-server ( -- )  S" HELO "       >STR  server 2@ STR+  sline ;
: mail-from   ( -- )  S" MAIL FROM: " >STR  mfrom  2@ STR+  sline ;
: rcpt-to     ( -- )  S" RCPT TO: "   >STR  mto    2@ STR+  sline ;
: data        ( -- )  S" DATA"        >STR                  sline ;
: send-quit   ( -- )  S" QUIT"        >STR                  sline ;
: mail-exit   ( -- )  mfile CLOSE-FILE DROP  msocket CLOSE-SOCKET ;

: send-file ( -- )
	.data-header sline   \ 2 carriage returns
	BEGIN  PAD DUP 4096 mfile READ-LINE ?FILE 
	WHILE  >STR sline
	REPEAT 2DROP
	S" " >STR +CR S" ." STR+ sline ;

: mail-loop ( -- )  helo-server  mail-from  rcpt-to  data  send-file  send-quit ;
: SENDMAIL ( "server" "to" "subject" "file" -- ) mail-init  mail-loop  mail-exit ;

: .ABOUT  
	CR .` Usage:   SENDMAIL mailserv to_addr subject messagefile`
	CR .` Example: S" smtp.chello.nl" S" mhx@iae.nl" S" Send mail works!" S" ./file.txt" SENDMAIL` ;

	.ABOUT CR

