\ socket interface

\ Copyright (C) 1998,2000,2003 Free Software Foundation, Inc.

\ This file is part of Gforth.

\ Gforth is free software; you can redistribute it and/or
\ modify it under the terms of the GNU General Public License
\ as published by the Free Software Foundation; either version 2
\ of the License, or (at your option) any later version.

\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\ GNU General Public License for more details.

\ You should have received a copy of the GNU General Public License
\ along with this program; if not, write to the Free Software
\ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.

require lib.fs
[IFUNDEF] libc  library libc libc.so.6  [THEN]

1 (int) libc gethostbyname gethostbyname ( name -- hostent )
3 (int) libc socket socket ( class type proto -- fd )
3 (int) libc connect connect ( fd sock size -- err )
2 (int) libc fdopen fdopen ( fd fileattr -- file )
1 (int) libc htonl htonl ( x -- x' )

struct
    cell% field h_name
    cell% field h_aliases
    cell% field h_addrtype
    cell% field h_length
    cell% field h_addr_list
end-struct hostent

struct
    cell% field family+port
    cell% field sin_addr
    cell% 2* field padding
end-struct sockaddr_in

Create sockaddr-tmp
sockaddr-tmp sockaddr_in %size dup allot erase

: c-string ( addr u -- addr' )
    tuck pad swap move pad + 0 swap c! pad ;

: host>addr ( addr u -- x )
    \G converts a internet name into a IPv4 address
    \G the resulting address is in network byte order
    c-string gethostbyname dup 0= abort" address not found"
    h_addr_list @ @ @ ;

2 Constant PF_INET
1 Constant SOCK_STREAM
6 Constant IPPROTO_TCP

: open-socket ( addr u port -- fid )
    htonl PF_INET [ base c@ 0= ] [IF] $10 lshift [THEN]
    or sockaddr-tmp family+port !
    host>addr sockaddr-tmp sin_addr !
    PF_INET SOCK_STREAM IPPROTO_TCP socket
    dup 0<= abort" no free socket" >r
    r@ sockaddr-tmp $10 connect 0< abort" can't connect"
    r> ;

\ -----------------------------------------------------------------------------------------
   4 CONSTANT F_SETFL
$100 CONSTANT MSG_WAITALL
$802 CONSTANT O_NONBLOCK|O_RDWR
2000 VALUE    SOCKET-TIMEOUT
CREATE CRLF 2 c, 13 c, 10 c,

1 (int) libc closesocket close          ( socket -- ior )
4 (int) libc send        send           ( socket buffer count flags -- size )
4 (int) libc recv        recv           ( socket buffer count flags -- size )
3 (int) libc fcntl 	 fcntl 		( fd n1 n2 -- ior )
2 (int) libc gethostname gethostname 	( c-addr u -- ior )

: strlen ( addr -- count ) 0 SWAP BEGIN  COUNT  WHILE  SWAP 1+ SWAP  REPEAT DROP ;
CREATE hostname$ 256 CHARS ALLOT
PAD 1+ 255 gethostname DROP  PAD 1+ strlen PAD C!  PAD hostname$ PAD C@ 1+ CMOVE

: MS@ ( -- u ) utime 1000 UM/MOD NIP ;
: BLOCKING-MODE ( socket flag -- ) F_SETFL SWAP IF  0  ELSE  O_NONBLOCK|O_RDWR  THEN  fcntl 0< ABORT" blocking-mode failed" ;
: HOSTNAME ( -- c-addr u ) hostname$ COUNT ;
: OPEN-SERVICE ( c-addr u port# -- socket ) open-socket ;
: SET-SOCKET-TIMEOUT ( u -- ) 200 + TO SOCKET-TIMEOUT ;
: GET-SOCKET-TIMEOUT ( -- u ) SOCKET-TIMEOUT 200 - ;

: (RS)  ( socket c-addr maxlen -- c-addr size ior ) 
	2 PICK >R R@ FALSE BLOCKING-MODE
	OVER >R MSG_WAITALL recv DUP 0 MAX 
	R> SWAP ROT
	R> TRUE BLOCKING-MODE ;

: READ-SOCKET 
	MS@ SOCKET-TIMEOUT + { socket c-addr maxlen tmax -- c-addr size }
	BEGIN 
	   socket c-addr maxlen (RS) 0<=
	   MS@ tmax U< AND 
	WHILE 
  	   2DROP
	REPEAT ;

: WRITE-SOCKET ( c-addr size socket -- ) -ROT 0 send 0< THROW ;
: CLOSE-SOCKET ( socket -- ) closesocket DROP ;
: PLACE ( c-addr1 u1 c-addr2 -- ) SWAP CMOVE ;

: STR+ { c-addr1 u1 c-addr2 u2 -- c-addr3 u3 }
	u1 u2 + ALLOCATE THROW 
	c-addr1 u1  2 PICK       PLACE 
	c-addr2 u2  2 PICK u1 +  PLACE  
	u1 u2 + ;

: >STR ( c-addr1 u1 -- c-addr2 u2 ) CRLF 0     STR+ ;
: +CR  ( c-addr1 u1 -- c-addr2 u2 ) CRLF COUNT STR+ ;

\ Some tests ----------
: .QUOTE ( -- )
	HOSTNAME 17 open-service ( -- socket )
	DUP PAD 2000 read-socket CR TYPE
	close-socket ;

: .TIME-OF-DAY ( -- )
	HOSTNAME 13 open-service ( -- socket )
	DUP PAD 200 read-socket CR TYPE
	close-socket ;

